/*
    This file is part of Extend Panel Menu

    Extend Panel Menu is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Extend Panel Menu is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Extend Panel Menu.  If not, see <http://www.gnu.org/licenses/>.

    Copyright 2017 Julio Galvan
*/
const Lang = imports.lang;
const Gio = imports.gi.Gio;
const Main = imports.ui.main;
const St = imports.gi.St;
const Mainloop = imports.mainloop;
const Shell = imports.gi.Shell;
const Config = imports.misc.config;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;

const Utils = Me.imports.indicators.utils;
const MenuItems = Me.imports.menuItems.MenuItems;
const CustomButton = Me.imports.indicators.button.CustomButton;
const CalendarIndicator = Me.imports.indicators.calendar.CalendarIndicator;
const NetworkIndicator = Me.imports.indicators.network.NetworkIndicator;
const NotificationIndicator = Me.imports.indicators.notification.NotificationIndicator;
const PowerIndicator = Me.imports.indicators.power.PowerIndicator;
const VolumeIndicator = Me.imports.indicators.volume.VolumeIndicator;
const DriveMenu = Me.imports.indicators.driveMenu.DriveMenu;

function init() {
    Convenience.initTranslations("kiran-panel-menu");
}

let settings;
let menuItems;
let indicators;
let settingsChanged;
let powertype;

let volume;
let network;
let power;
let calendar;
let notification;
let driveMenu;

let tracker = Shell.WindowTracker.get_default();
let _showDesktopButton;
let _focusAppChangeId = null;
let _restoreWindowList = null;;

const BUS_NAME = 'org.freedesktop.UPower';
const OBJECT_PATH = '/org/freedesktop/UPower/devices/DisplayDevice';

const DisplayDeviceInterface = '<node> \
<interface name="org.freedesktop.UPower.Device"> \
  <property name="Type" type="u" access="read"/> \
  <property name="State" type="u" access="read"/> \
  <property name="Percentage" type="d" access="read"/> \
  <property name="TimeToEmpty" type="x" access="read"/> \
  <property name="TimeToFull" type="x" access="read"/> \
  <property name="IsPresent" type="b" access="read"/> \
  <property name="IconName" type="s" access="read"/> \
</interface> \
</node>';

const PowerManagerProxy = Gio.DBusProxy.makeProxyWrapper(DisplayDeviceInterface);

const CENTER_BOX = Main.panel._centerBox;
const RIGHT_BOX = Main.panel._rightBox;

function enable() {
    Main.panel.statusArea.aggregateMenu.container.hide();
    Main.panel.statusArea.dateMenu.container.hide();
    Main.panel._rightBox.remove_child(Main.panel.statusArea.dateMenu.container);
    _proxy = new PowerManagerProxy(Gio.DBus.system, BUS_NAME, OBJECT_PATH);
    powertype = _proxy.IsPresent;

    network = new NetworkIndicator();
    volume = new VolumeIndicator();
    if (powertype)
        power = new PowerIndicator();
    calendar = new CalendarIndicator();
    notification = new NotificationIndicator();
    driveMenu = new DriveMenu();

    _showDesktopButton = new St.Bin({ style_class: 'showdesktop-button',
                            reactive: true,
                            can_focus: true,
                            x_fill: true,
                            y_fill: true,
                            track_hover: true });
    _showDesktopButton.connect('button-release-event', Lang.bind(this, _onShowDesktopButtonPress));
    _showDesktopButton.connect('enter-event', Lang.bind(this, function(){
                _showDesktopButton.add_style_class_name('showdesktop-button-hovered');
            }));

    _showDesktopButton.connect('leave-event', Lang.bind(this, function(){
                _showDesktopButton.remove_style_class_name('showdesktop-button-hovered');
            }));

    let position = 6;

    if (powertype) {
        Main.panel._rightBox.insert_child_at_index(_showDesktopButton, position+1);
        Main.panel.addToStatusArea(notification.name, notification, position, "right");
        Main.panel.addToStatusArea(calendar.name, calendar, position-1, "right");
        Main.panel.addToStatusArea(power.name, power, position-2, "right");
        Main.panel.addToStatusArea(network.name, network, position-3, "right");
        Main.panel.addToStatusArea(volume.name, volume, position-4, "right");
        Main.panel.addToStatusArea(driveMenu.name, driveMenu, position-5, "right");
    } else {
        Main.panel._rightBox.insert_child_at_index(_showDesktopButton, position);
        Main.panel.addToStatusArea(notification.name, notification, position-1, "right");
        Main.panel.addToStatusArea(calendar.name, calendar, position-2, "right");
        Main.panel.addToStatusArea(network.name, network, position-3, "right");
        Main.panel.addToStatusArea(volume.name, volume, position-4, "right");
        Main.panel.addToStatusArea(driveMenu.name, driveMenu, position-5, "right");
    }

    // Load Settings
    settings = Convenience.getSettings();
    menuItems = new MenuItems(settings);
    settingsChanged = new Array();
    settingsChanged[0] = settings.connect("changed::items", Lang.bind(this, applySettings));
    settingsChanged[1] = settings.connect("changed::tray-offset", Lang.bind(this, applySettings));
    settingsChanged[2] = settings.connect("changed::spacing", Lang.bind(this, changeSpacing));
    settingsChanged[5] = settings.connect("changed::date-format", Lang.bind(this, changeDateformat));
    settingsChanged[6] = settings.connect("changed::autohide-notification", Lang.bind(this, changeAutohide));
    if (powertype) {
        settingsChanged[7] = settings.connect("changed::autohide-on-full-power", Lang.bind(this, changeFullPowerHide));
        settingsChanged[8] = settings.connect("changed::autohide-on-percent", Lang.bind(this, changePowerHide));
        settingsChanged[9] = settings.connect("changed::autohide-when-percent", Lang.bind(this, changePowerHide));
        settingsChanged[10] = settings.connect("changed::autohide-power-icon-label", Lang.bind(this, changePowerHide));
        changeFullPowerHide();
        changePowerHide();
    }
    applySettings();
    changeSpacing();
    changeDateformat();
    changeAutohide();

}

function _onShowDesktopButtonPress() {
        if(_focusAppChangeId){
            tracker.disconnect(_focusAppChangeId);
            _focusAppChangeId = null;
        }

        if(_restoreWindowList && _restoreWindowList.length) {
            let current_workspace = Utils.DisplayWrapper.getWorkspaceManager().get_active_workspace();
            let windows = current_workspace.list_windows();
            _restoreWindowList.forEach(function(w) {
                if(windows.indexOf(w) > -1)
                    Main.activateWindow(w);
            });
            _restoreWindowList = null;

            Main.overview.hide();
        } else {
            let current_workspace = Utils.DisplayWrapper.getWorkspaceManager().get_active_workspace();
            let windows = current_workspace.list_windows().filter(function (w) {
                return w.showing_on_its_workspace() && !w.skip_taskbar;
            });
            windows = global.display.sort_windows_by_stacking(windows);

            windows.forEach(function(w) {
                w.minimize();
            });

            _restoreWindowList = windows;

            Mainloop.timeout_add(0, Lang.bind(this, function () {
                _focusAppChangeId = tracker.connect('notify::focus-app', Lang.bind(this, function () {
                    _restoreWindowList = null;
                }));
            }));

            Main.overview.hide();
        }
}

function versionCheck(required, current) {
	let currentArray = current.split('.');
	let major = currentArray[0];
	let minor = currentArray[1];
	let requiredArray = required.split('.');
	if (requiredArray[0] == major && parseInt(requiredArray[1]) <= parseInt(minor))
		return true;
	return false;
}

function changeSpacing() {
    let spacing = settings.get_int("spacing");
    indicators.forEach(function (item) {
        item.set_spacing(spacing);
    });
}
function changeDateformat() {
    let dateformat = settings.get_string("date-format");
    calendar.override(dateformat);
}
function changeAutohide() {
    let autoHideNotification = settings.get_boolean("autohide-notification");
    notification.setHide(autoHideNotification);
}
function changeFullPowerHide() {
    let hideOnFull = settings.get_boolean("autohide-on-full-power");
    power.setHideOnFull(hideOnFull);
}
function changePowerHide() {
    let hideOnPercent = settings.get_boolean("autohide-on-percent");
    let hideWhenPercent = settings.get_int("autohide-when-percent");
    let hideElements = settings.get_int("autohide-power-icon-label");
    power.setHideOnPercent(hideOnPercent, hideWhenPercent, hideElements);
}

function applySettings() {
    let enabled = menuItems.getEnableItems();
    let center = menuItems.getCenterItems();
    indicators = new Array(enabled.length);

    removeAll();
    if (powertype)
        setup(enabled, center, indicators, "power", power);
    setup(enabled, center, indicators, "volume", volume);
    setup(enabled, center, indicators, "network", network);
    setup(enabled, center, indicators, "notification", notification);
    setup(enabled, center, indicators, "calendar", calendar);

    let rightchildren = RIGHT_BOX.get_children().length;
    let centerchildren = CENTER_BOX.get_children().length;

    let spacing = settings.get_int("spacing");
    let offset = settings.get_int("tray-offset");

    indicators.reverse().forEach(function (item) {
        item.set_spacing(spacing);
        if (item._center) {
            CENTER_BOX.insert_child_at_index(item.container, centerchildren);
        } else {
            RIGHT_BOX.insert_child_at_index(item.container, rightchildren - offset);
        }
    });

}

function setup(enabledItems, centerItems, arrayIndicators, name, indicator) {
    let index = enabledItems.indexOf(name);
    let valid = index != -1;
    if (valid) {
        arrayIndicators[index] = indicator;
        arrayIndicators[index]._center = centerItems.indexOf(name) != -1;
    }
}

function removeAll() {
    removeContainer(volume);
    removeContainer(network);
    if (powertype)
        removeContainer(power);
    removeContainer(calendar);
    removeContainer(notification);
}

function removeContainer(item) {
    if (item._center) {
        CENTER_BOX.remove_child(item.container)
    } else {
        RIGHT_BOX.remove_child(item.container);
    }
    item._center = false;
}

function disable() {
    settingsChanged.forEach(function (item) {
        settings.disconnect(item);
    });
    settingsChanged = null;
    settings = null;
    
    Main.panel._rightBox.remove_child(_showDesktopButton);
    _showDesktopButton.destroy();
    _showDesktopButton = null;

    volume.destroy();
    if (powertype)
        power.destroy();
    network.destroy();
    notification.destroy();
    calendar.destroy();
    driveMenu.destroy();

    Main.panel.statusArea.aggregateMenu.container.show();
    Main.panel.statusArea.dateMenu.container.show();
    Main.panel._rightBox.add_child(Main.panel.statusArea.dateMenu.container);
}
